@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,11 +27,12 @@
  *
  * For details, see Luis, J.F. and M.C. Neves. 2006, "The isostatic compensation of the Azores Plateau:
  * a 3D admittance and coherence analysis. J. Geotermal Vulc. Res. Volume 156, Issues 1-2 ,
- * Pages 10-22, http://dx.doi.org/10.1016/j.jvolgeores.2006.03.010
+ * Pages 10-22, https://doi.org/10.1016/j.jvolgeores.2006.03.010
  *
  * */
 
 #include "gmt_dev.h"
+#include "newton.h"
 
 #define THIS_MODULE_CLASSIC_NAME	"gravfft"
 #define THIS_MODULE_MODERN_NAME	"gravfft"
@@ -77,7 +78,7 @@ struct GRAVFFT_CTRL {
 	struct GRAVFFT_F {	/* -F[f[+s]|b|g|e|n|v] */
 		bool active;
 		bool slab;
-		bool bouger;
+		bool bouguer;
 		unsigned int mode;
 	} F;
 	struct GRAVFFT_G {	/* -G<outfile> */
@@ -129,7 +130,6 @@ struct GRAVFFT_CTRL {
 #define FSIGNIF 24
 #endif
 
-#define GRAVITATIONAL_CONST 6.667e-11
 #define	YOUNGS_MODULUS	7.0e10		/* Pascal = Nt/m**2  */
 #define	NORMAL_GRAVITY	9.806199203	/* Moritz's 1980 IGF value for gravity at 45 degrees latitude (m/s) */
 #define	POISSONS_RATIO	0.25
@@ -178,11 +178,11 @@ GMT_LOCAL int gravfft_do_admittance(struct GMT_CTRL *GMT, struct GMT_GRID *Grid,
 static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OPTION *options) {
 
 	unsigned int n_errors = 0;
-
 	int n, override_mode = GMT_FFT_REMOVE_MID;
 	struct GMT_OPTION *opt = NULL,  *popt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
 	char   ptr[GMT_BUFSIZ] = {""}, t_or_b[4] = {""}, argument[GMT_LEN16] = {""}, combined[GMT_BUFSIZ] = {""};
+
 	if (gmt_M_compat_check (GMT, 4)) {
 		char *mod = NULL;
 		if ((popt = GMT_Find_Option (API, 'L', options)) != 0) {	/* Gave old -L */
@@ -196,20 +196,19 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 	for (opt = options; opt; opt = opt->next) {		/* Process all the options given */
 		switch (opt->option) {
 
-			case '<':	/* Input file */
+			case '<':	/* Input file (repeatable up to 2 times) */
 				Ctrl->In.active = true;
 				if (Ctrl->In.n_grids >= 2) {
 					n_errors++;
 					GMT_Report (API, GMT_MSG_ERROR, "A maximum of two input grids may be processed\n");
 				}
 				else {
-					Ctrl->In.file[Ctrl->In.n_grids] = strdup (opt->arg);
-					if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->In.file[Ctrl->In.n_grids]))) n_errors++;;
+					n_errors += gmt_get_required_file (GMT, opt->arg, opt->option, 0, GMT_IS_GRID, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->In.file[Ctrl->In.n_grids]));
 					Ctrl->In.n_grids++;
 				}
 				break;
 			case 'C':	/* For theoretical curves only */
-				Ctrl->C.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->C.active);
 				sscanf (opt->arg, "%d/%lf/%lf/%s", &Ctrl->C.n_pt, &Ctrl->C.theor_inc, &Ctrl->misc.z_level, t_or_b);
 				for (n = 0; t_or_b[n]; n++) {
 					switch (t_or_b[n]) {
@@ -230,13 +229,13 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 				}
 				break;
 			case 'D':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->D.active);
 				if (!opt->arg) {
 					GMT_Report (API, GMT_MSG_ERROR,
 					            "Option -D: must give constant density contrast or grid with density contrasts\n");
 					n_errors++;
 				}
 				else {
-					Ctrl->D.active = true;
 					if (!gmt_access (GMT, opt->arg, R_OK)) {	/* Gave a grid with density contrast */
 						Ctrl->D.file = strdup (opt->arg);
 						Ctrl->D.variable = true;
@@ -248,7 +247,8 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 				}
 				break;
 			case 'E':
-				Ctrl->E.n_terms = atoi (opt->arg);
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->E.active);
+				n_errors += gmt_get_required_uint (GMT, opt->arg, opt->option, 0, &Ctrl->E.n_terms);
 				if (Ctrl->E.n_terms > 10) {
 					GMT_Report (API, GMT_MSG_ERROR, "Option -E: n_terms must be <= 10\n");
 					n_errors++;
@@ -259,7 +259,7 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 				}
 				break;
 			case 'F':
-				Ctrl->F.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->F.active);
 				switch (opt->arg[0]) {
 					case 'g': Ctrl->F.mode = GRAVFFT_GEOID;      break;
 					case 'v': Ctrl->F.mode = GRAVFFT_VGG;        break;
@@ -268,7 +268,7 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 					case 'b':
 						Ctrl->F.mode   = GRAVFFT_FAA;
 						Ctrl->F.slab   = true;
-						Ctrl->F.bouger = true;
+						Ctrl->F.bouguer = true;
 						break;
 					case 'f': default:
 						Ctrl->F.mode = GRAVFFT_FAA; 	   /* FAA */
@@ -277,12 +277,11 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 				}
 				break;
 			case 'G':
-				Ctrl->G.active = true;
-				if (opt->arg[0]) Ctrl->G.file = strdup (opt->arg);
-				if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->G.file))) n_errors++;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->G.active);
+				n_errors += gmt_get_required_file (GMT, opt->arg, opt->option, 0, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->G.file));
 				break;
 			case 'I':
-				Ctrl->I.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->I.active);
 				for (n = 0; opt->arg[n]; n++) {
 					switch (opt->arg[n]) {
 						case 'w':
@@ -311,10 +310,10 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 				if (gmt_M_compat_check (GMT, 4))
 					GMT_Report (API, GMT_MSG_COMPAT, "Option -L is deprecated; use -N modifiers in the future.\n");
 				else
-					n_errors += gmt_default_error (GMT, opt->option);
+					n_errors += gmt_default_option_error (GMT, opt);
 				break;
 			case 'N':
-				Ctrl->N.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->N.active);
 				if (popt && gmt_M_compat_check (GMT, 4)) {	/* Got both old -L and -N; append */
 					sprintf (combined, "%s%s", opt->arg, argument);
 					Ctrl->N.info = GMT_FFT_Parse (API, 'N', GMT_FFT_DIM, combined);
@@ -325,14 +324,16 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 				Ctrl->N.active = true;
 				break;
 			case 'Q':
-				Ctrl->Q.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->Q.active);
+				n_errors += gmt_get_no_argument (GMT, opt->arg, opt->option, 0);
 				override_mode = GMT_FFT_REMOVE_MID;	/* Leave trend alone and remove mid value */
 				break;
 			case 'S':
-				Ctrl->S.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->S.active);
+				n_errors += gmt_get_no_argument (GMT, opt->arg, opt->option, 0);
 				break;
 			case 'T':
-				Ctrl->T.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->T.active);
 				n = sscanf (opt->arg, "%lf/%lf/%lf/%lf/%lf", &Ctrl->T.te, &Ctrl->T.rhol, &Ctrl->T.rhom, &Ctrl->T.rhow, &Ctrl->T.rhoi);
 				Ctrl->T.rho_cw = Ctrl->T.rhol - Ctrl->T.rhow;
 				Ctrl->T.rho_mc = Ctrl->T.rhom - Ctrl->T.rhol;
@@ -348,15 +349,15 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 				}
 				break;
 			case 'W':	/* Water depth */
-				Ctrl->W.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->W.active);
 				GMT_Get_Values (API, opt->arg, &Ctrl->W.water_depth, 1);
 				break;
 			case 'Z':
-				Ctrl->Z.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->Z.active);
 				sscanf (opt->arg, "%lf/%lf", &Ctrl->Z.zm, &Ctrl->Z.zl);
 				break;
 			default:
-				n_errors += gmt_default_error (GMT, opt->option);
+				n_errors += gmt_default_option_error (GMT, opt);
 				break;
 		}
 	}
@@ -418,65 +419,72 @@ static int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_OP
 static int usage (struct GMTAPI_CTRL *API, int level) {
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: %s <topo_grd> [<ingrid2>] -G<outgrid> [-C<n/wavelength/mean_depth/tbw>]\n", name);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-D<density|grid>] [-E<n_terms>] [-F[f[+s]|b|g|v|n|e]] [-I<wbctk>]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-N%s] [-Q]\n", GMT_FFT_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-T<te/rl/rm/rw>[/<ri>][+m]] [%s] [-W<wd>] [-Z<zm>[/<zl>]] [-fg] [%s]\n\n", GMT_V_OPT, GMT_PAR_OPT);
+	GMT_Usage (API, 0, "usage: %s %s [<ingrid2>] -G%s [-C<n/wavelength/mean_depth/tbw>] "
+		"[-D<density>] [-E<n_terms>] [-F[f[+s]|b|g|v|n|e]] [-I<cbktw>] [-N%s] [-Q] [-S] "
+		"[-T<te/rl/rm/rw>[/<ri>][+m]] [%s] [-W<wd>[k]] [-Z<zm>[/<zl>]] [-fg] [%s]\n",
+		name, GMT_INGRID, GMT_OUTGRID, GMT_FFT_OPT, GMT_V_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
-	GMT_Message (API, GMT_TIME_NONE, "\ttopo_grd is the input grdfile with topography values\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-G Filename for output netCDF grdfile with gravity [or geoid] values\n");
-	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-C Compute admittance curves based on a theoretical model.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append <n/wavelength/mean_depth/tbw> as specified below:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Total profile length in meters = <n> * <wavelength> (unless -Kx is set).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   --> Rest of parameters are set within -T AND -Z options\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append dataflags (one or two) of tbw:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     t writes \"elastic plate\" admittance.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     b writes \"loading from below\" admittance.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     w writes wavelength instead of wavenumber.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-D Sets density contrast across surface (used when not -T).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Give a co-registered density grid for a variable density contrast [constant].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-I Use <ingrid2> and <topo_grd> to estimate admittance|coherence and write\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   it to stdout (-G ignored if set). This grid should contain gravity or geoid\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   for the same region of <topo_grd>. Default computes admittance. Output\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   contains 3 or 4 columns. Frequency (wavelength), admittance (coherence)\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   one sigma error bar and, optionally, a theoretical admittance.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append dataflags (one to three) of wbct:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     w writes wavelength instead of wavenumber.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     k Use km or wavelength unit [m].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     c computes coherence instead of admittance.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     b writes a forth column with \"loading from below\" \n");
-	GMT_Message (API, GMT_TIME_NONE, "\t       theoretical admittance.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     t writes a forth column with \"elastic plate\" \n");
-	GMT_Message (API, GMT_TIME_NONE, "\t       theoretical admittance.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-E Number of terms used in Parker's expansion [Default = 3].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-F Specify desired geopotential field:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   b = Bouguer anomalies (mGal).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   f = Free-air anomalies (mGal) [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t       Append +s to adjust for implied slab correction [none].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   g = Geoid anomalies (m).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   v = Vertical Gravity Gradient (VGG; 1 Eovtos = 0.1 mGal/km).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   e = East deflections of the vertical (micro-radian).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   n = North deflections of the vertical (micro-radian).\n");
+	GMT_Message (API, GMT_TIME_NONE, "  REQUIRED ARGUMENTS:\n");
+	gmt_ingrid_syntax (API, 0, "Name of input grid with topography values.  Optionally, give a second grid <ingrid2> for cross-spectral computations (same modifiers apply)");
+	gmt_outgrid_syntax (API, 'G', "Set name of the output grid file with gravity [or geoid] value");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
+	GMT_Usage (API, 1, "\n-C<n/wavelength/mean_depth/tbw>");
+	GMT_Usage (API, -2, "Compute admittance curves based on a theoretical model. "
+		"Append <n/wavelength/mean_depth/tbw> as specified below. "
+		"Total profile length in meters = <n> * <wavelength>. "
+		"The remaining parameters are set by -T and -Z options. "
+		"Append /dataflags (one or two) from tbw:");
+	GMT_Usage (API, 3, "b: Write \"loading from below\" admittance.");
+	GMT_Usage (API, 3, "t: Write \"elastic plate\" admittance.");
+	GMT_Usage (API, 3, "w: Write wavelength instead of wavenumber.");
+	GMT_Usage (API, 1, "\n-D<density>");
+	GMT_Usage (API, -2, "Set density contrast across surface (used when not -T). "
+		"Append a co-registered density grid for a variable density contrast or a constant <density>.");
+	GMT_Usage (API, 1, "\n-E<n_terms>");
+	GMT_Usage (API, -2, "Number of terms used in Parker's expansion [Default = 3].");
+	GMT_Usage (API, 1, "\n-F[f[+s]|b|g|v|n|e]");
+	GMT_Usage (API, -2, "Specify desired geopotential field:");
+	GMT_Usage (API, 3, "b: Bouguer anomalies (mGal).");
+	GMT_Usage (API, 3, "f: Free-air anomalies (mGal) [Default].");
+	GMT_Usage (API, 4, "+s Adjust for implied slab correction [none].");
+	GMT_Usage (API, 3, "g: Geoid anomalies (m).");
+	GMT_Usage (API, 3, "v: Vertical Gravity Gradient (VGG; 1 Eovtos = 0.1 mGal/km).");
+	GMT_Usage (API, 3, "e: East deflections of the vertical (micro-radian).");
+	GMT_Usage (API, 3, "n: North deflections of the vertical (micro-radian).");
+	GMT_Usage (API, 1, "\n-I<cbktw>");
+	GMT_Usage (API, -2, "Use <ingrid2> and <ingrid> to estimate admittance|coherence and write "
+		"output to standard output (-G ignored if set). The <ingrid2>  should contain gravity or geoid anomalies "
+		"for the same region of <ingrid>. Default computes admittance. Output "
+		"contains 3 or 4 columns: Frequency (wavelength), admittance (coherence) "
+		"one sigma error bar and, optionally, a theoretical admittance. "
+		"Append dataflags (one to three) from cbktw:");
+	GMT_Usage (API, 3, "c: Compute coherence instead of admittance.");
+	GMT_Usage (API, 3, "b: Write a forth column with \"loading from below\" theoretical admittance.");
+	GMT_Usage (API, 3, "k: Use km or wavelength unit [m].");
+	GMT_Usage (API, 3, "t: Write a forth column with \"elastic plate\" theoretical admittance.");
+	GMT_Usage (API, 3, "w: Write wavelength instead of wavenumber.");
 	GMT_FFT_Option (API, 'N', GMT_FFT_DIM, "Choose or inquire about suitable grid dimensions for FFT, and set modifiers.");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Warning: both -D -T...+m and -Q will implicitly set -N's +h.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-Q Writes out a grid with the flexural topography (with z positive up)\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   whose average depth is set to the value given by -Z<zm>.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-S Computes predicted geopotential (see -F) grid due to a subplate load\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   produced by the current bathymetry and the theoretical admittance.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   The necessary parameters are set within -T and -Z options\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-T Computes the isostatic compensation. Input file is topo load.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append elastic thickness and densities of load, mantle, and\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   water, all in SI units. Give average mantle depth via -Z\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   If the elastic thickness is > 1e10 it will be interpreted as the\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   flexural rigidity (by default it is computed from Te and Young modulus).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   If an optional infill density <ri> != <rl> is appended we find an approximate solution.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Optionally, append +m to write a grid with the Moho's geopotential effect\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   (see -F) from model selected by -T.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-W Specify water depth (or observation level) in m; append k for km.  Must be positive.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-Z Specify Moho [and swell] average compensation depths.\n");
+	GMT_Usage (API, -2, "Warning: both -D -T...+m and -Q will implicitly set -N's +h.");
+	GMT_Usage (API, 1, "\n-Q Writes out a grid with the flexural topography (with z positive up) "
+		"whose average depth is set to the value given by -Z<zm>.");
+	GMT_Usage (API, 1, "\n-S Computes predicted geopotential (see -F) grid due to a subplate load "
+		"produced by the current bathymetry and the theoretical admittance. "
+		"The necessary parameters are set within -T and -Z options.");
+	GMT_Usage (API, 1, "\n-T<te/rl/rm/rw>[/<ri>][+m]");
+	GMT_Usage (API, -2, "Computes the isostatic compensation. Input file is topo load. "
+		"Append elastic thickness and densities of load, mantle, and "
+		"water, all in SI units. Give average mantle depth via -Z. "
+		"If the elastic thickness is > 1e10 it will be interpreted as the "
+		"flexural rigidity [by default it is computed from Te and Young modulus]. "
+		"If an optional infill density <ri> != <rl> is appended we find an approximate solution.");
+	GMT_Usage (API, 3, "+m Write a grid with the Moho's geopotential effect "
+		"(see -F) from model selected by -T.");
+	GMT_Usage (API, 1, "\n-W<wd>[k]");
+	GMT_Usage (API, -2, "Specify water depth (or observation level) in m; append k for km.  Must be positive.");
+	GMT_Usage (API, 1, "\n-Z<zm>[/<zl>]");
+	GMT_Usage (API, -2, "Specify Moho [and swell] average compensation depths.");
 	GMT_Option (API, "V");
 	GMT_Message (API, GMT_TIME_NONE, "\t-fg Convert geographic grids to meters using a \"Flat Earth\" approximation.\n");
 	GMT_Option (API, ".");
@@ -494,6 +502,7 @@ EXTERN_MSC int GMT_gravfft (void *V_API, int mode, void *args) {
 	double   delta_pt, freq;
 
 	struct GMT_GRID *Grid[2] = {NULL, NULL}, *Orig[2] = {NULL, NULL}, *Rho = NULL;
+	struct GMT_GRID_HEADER_HIDDEN *HH = NULL;
 	struct GMT_FFT_WAVENUMBER *FFT_info[2] = {NULL, NULL}, *K = NULL, *Rho_info = NULL;
 	struct GRAVFFT_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
@@ -615,6 +624,7 @@ EXTERN_MSC int GMT_gravfft (void *V_API, int mode, void *args) {
 			GMT_Report (API, GMT_MSG_ERROR, "Surface and density grids have different intervals\n");
 			Return (GMT_RUNTIME_ERROR);
 		}
+		for (m = 0; m < Rho->header->size; m++) if (gmt_M_is_fnan (Rho->data[m])) Rho->data[m] = Rho->header->z_min;	/* Replace any NaNs with the minimum density */
 	}
 
 	/* Grids are compatible. Initialize FFT structs, grid headers, read data, and check for NaNs */
@@ -624,6 +634,11 @@ EXTERN_MSC int GMT_gravfft (void *V_API, int mode, void *args) {
 		if ((Orig[k] = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY |
                                       GMT_GRID_IS_COMPLEX_REAL, NULL, Ctrl->In.file[k], Orig[k])) == NULL)	/* Get data only */
 			Return (API->error);
+		HH = gmt_get_H_hidden (Orig[k]->header);
+		if (HH->has_NaNs == GMT_GRID_HAS_NANS) {
+			GMT_Report (API, GMT_MSG_ERROR, "Input grid %s has NaNs, cannot be used with FFTs\n", Ctrl->In.file[k]);
+			Return (GMT_RUNTIME_ERROR);
+		}
 		/* Note: If input grid(s) are read-only then we must duplicate them; otherwise Grid[k] points to Orig[k] */
 		(void) gmt_set_outgrid (GMT, Ctrl->In.file[k], false, 0, Orig[k], &Grid[k]);
 	}
@@ -632,7 +647,7 @@ EXTERN_MSC int GMT_gravfft (void *V_API, int mode, void *args) {
 	 * we are done with using the addresses Orig[k] directly. */
 
 	if (Ctrl->W.active) {	/* Need to adjust for a different observation level relative to topo.grd */
-		unsigned int row, col;
+		openmp_int row, col;
 		GMT_Report (API, GMT_MSG_INFORMATION, "Remove %g m from topography grid %s\n", Ctrl->W.water_depth, Ctrl->In.file[0]);
 		gmt_M_grd_loop (GMT, Grid[0], row, col, m)
 			Grid[0]->data[m] -= (gmt_grdfloat)Ctrl->W.water_depth;
@@ -709,8 +724,8 @@ EXTERN_MSC int GMT_gravfft (void *V_API, int mode, void *args) {
 			/* The data are in the middle of the padded array; only the interior (original dimensions) will be written to file */
 			if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Grid[0]))
 				Return (API->error);
-			if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY |
-                                GMT_GRID_IS_COMPLEX_REAL, NULL, Ctrl->G.file, Grid[0]) != GMT_NOERROR) {
+			if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY,
+                                NULL, Ctrl->G.file, Grid[0]) != GMT_NOERROR) {
 				Return (API->error);
 			}
 			GMT_FFT_Destroy (API, &(FFT_info[0]));
@@ -768,9 +783,6 @@ EXTERN_MSC int GMT_gravfft (void *V_API, int mode, void *args) {
 		Return (GMT_RUNTIME_ERROR);
 	}
 
-	/* Manually demux back since we may do loops below */
-	gmt_grd_mux_demux (API->GMT, Grid[0]->header, Grid[0]->data, GMT_GRID_IS_SERIAL);
-
 	if (!doubleAlmostEqual (scale_out, 1.0))
 		gmt_scale_and_offset_f (GMT, Grid[0]->data, Grid[0]->header->size, scale_out, 0);
 
@@ -779,10 +791,11 @@ EXTERN_MSC int GMT_gravfft (void *V_API, int mode, void *args) {
 			strcpy (Grid[0]->header->title, "Gravity anomalies");
 			strcpy (Grid[0]->header->z_units, "mGal");
 			if (Ctrl->F.slab) {	/* Do the slab adjustment */
-				slab_gravity = (gmt_grdfloat) (1.0e5 * 2 * M_PI * Ctrl->misc.rho * GRAVITATIONAL_CONST *
+				if (Ctrl->D.variable) Ctrl->misc.rho = Rho->header->z_min;
+				slab_gravity = (gmt_grdfloat) (1.0e5 * 2 * M_PI * Ctrl->misc.rho * NEWTON_G *
 				                        fabs (Ctrl->W.water_depth - Ctrl->misc.z_level));
 				GMT_Report (API, GMT_MSG_INFORMATION, "Add %g mGal to predicted FAA grid to account for implied slab\n", slab_gravity);
-				if (Ctrl->F.bouger)		/* The complete bouger contribution */
+				if (Ctrl->F.bouguer)		/* The complete Bouguer contribution */
 					for (m = 0; m < Grid[0]->header->size; m++) Grid[0]->data[m] = slab_gravity - Grid[0]->data[m];
 				else
 					for (m = 0; m < Grid[0]->header->size; m++) Grid[0]->data[m] += slab_gravity;
@@ -820,8 +833,8 @@ EXTERN_MSC int GMT_gravfft (void *V_API, int mode, void *args) {
 		gmt_M_free (GMT, topo);
 		Return (API->error);
 	}
-	if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY |
-                        GMT_GRID_IS_COMPLEX_REAL, NULL, Ctrl->G.file, Grid[0]) != GMT_NOERROR) {
+	if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY,
+                        NULL, Ctrl->G.file, Grid[0]) != GMT_NOERROR) {
 		gmt_M_free (GMT, topo);
 		Return (API->error);
 	}
@@ -888,7 +901,7 @@ GMT_LOCAL void gravfft_do_parker (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, s
 	for (i = 2; i <= n; i++) f *= i;	/* n! */
 	p = n - 1.0;
 
-	c = 1.0e5 * 2.0 * M_PI * GRAVITATIONAL_CONST * rho / f; /* Gives mGal */
+	c = 1.0e5 * 2.0 * M_PI * NEWTON_G * rho / f; /* Gives mGal */
 
 	for (k = 0; k < Grid->header->size; k+= 2) {
 		mk = gmt_fft_get_wave (k, K);
@@ -1112,7 +1125,7 @@ GMT_LOCAL void gravfft_load_from_below_admitt(struct GMT_CTRL *GMT, struct GRAVF
 	for (k = 0; k < nk; k++) {
 		freq = (k + 1) * delta_k;
 		earth_curvature = (sphericity) ? (2 * earth_rad * freq) / (4 * M_PI * earth_rad * freq + 1) : 1.;
-		t1 = earth_curvature * (twopi * GRAVITATIONAL_CONST);
+		t1 = earth_curvature * (twopi * NEWTON_G);
 		if (Ctrl->F.mode == GRAVFFT_FAA)
 			t1 *= 1.0e5;     /* to have it in mGals */
 		else                 /* Must be the GEOID case */
@@ -1149,7 +1162,7 @@ GMT_LOCAL void gravfft_load_from_top_admitt (struct GMT_CTRL *GMT, struct GRAVFF
 	for (k = 0; k < nk; k++) {
 		freq = (k + 1) * delta_k;
 		earth_curvature = (sphericity) ? (2 * earth_rad * freq) / (4 * M_PI * earth_rad * freq + 1) : 1.;
-		t1 = earth_curvature * (twopi * GRAVITATIONAL_CONST);
+		t1 = earth_curvature * (twopi * NEWTON_G);
 		if (Ctrl->F.mode == GRAVFFT_FAA)
 			t1 *= 1.0e5;     /* to have it in mGals */
 		else                 /* Must be the GEOID case */
@@ -1189,7 +1202,7 @@ GMT_LOCAL void gravfft_load_from_top_grid (struct GMT_CTRL *GMT, struct GMT_GRID
 		else
 			t = pow (mk, p);
 		earth_curvature = (sphericity) ? (2 * earth_rad * mk) / (4 * M_PI * earth_rad * mk + 1) : 1.;
-		t1 = earth_curvature * (twopi * GRAVITATIONAL_CONST);
+		t1 = earth_curvature * (twopi * NEWTON_G);
 		if (Ctrl->F.mode == GRAVFFT_FAA)
 			t1 *= 1.0e5;     /* to have it in mGals */
 		else                 /* Must be the GEOID case */
@@ -1230,7 +1243,7 @@ GMT_LOCAL void gravfft_load_from_below_grid (struct GMT_CTRL *GMT, struct GMT_GR
 		else
 			t = pow (mk, p);
 		earth_curvature = (sphericity) ? (2 * earth_rad * mk) / (4 * M_PI * earth_rad * mk + 1) : 1.;
-		t1 = earth_curvature * (twopi * GRAVITATIONAL_CONST);
+		t1 = earth_curvature * (twopi * NEWTON_G);
 		if (Ctrl->F.mode == GRAVFFT_FAA)
 			t1 *= 1.0e5;     /* to have it in mGals */
 		else                 /* Must be the GEOID case */
