@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: grdredpol.c 18334 2017-06-07 19:25:56Z pwessel $
  *
- *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * API functions to support the grdredpol application.
@@ -21,72 +20,78 @@
  * Brief synopsis: grdredpol reads a grid file with the magnetic field anomaly and
  * computes the continuous reduction to the pole (RTP) anomaly by calculating the
  * filter coefficients in the frequency, inverse FT and convolve in space domain.
- * For details on the method see, Luis, J.F and J.M. Miranda (2008), 
- * "Reevaluation of magnetic chrons in the North Atlantic between 35N and 47N: 
- * Implications for the formation of the Azores Triple Junction and associated plateau, 
- * J. Geophys. Res., 113, B10105, doi:10.1029/2007JB005573 
+ * For details on the method see, Luis, J.F and J.M. Miranda (2008),
+ * "Reevaluation of magnetic chrons in the North Atlantic between 35N and 47N:
+ * Implications for the formation of the Azores Triple Junction and associated plateau,
+ * J. Geophys. Res., 113, B10105, doi:10.1029/2007JB005573
  *
  * Author:	Joaquim Luis / Miguel Miranda
  * Date:	18-Feb-2003 (original GMT4 version)
- * Version:	5 API
+ * Version:	6 API
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"grdredpol"
+#define THIS_MODULE_CLASSIC_NAME	"grdredpol"
+#define THIS_MODULE_MODERN_NAME	"grdredpol"
 #define THIS_MODULE_LIB		"potential"
 #define THIS_MODULE_PURPOSE	"Compute the Continuous Reduction To the Pole, AKA differential RTP"
 #define THIS_MODULE_KEYS	"<G{,EG(,GG},ZG)"
 #define THIS_MODULE_NEEDS	"g"
 #define THIS_MODULE_OPTIONS "-RVn"
 
-struct REDPOL_CTRL {
-	struct In {
+struct GRDREDPOL_CTRL {
+	struct GRDREDPOL_In {
 		bool active;
 		char *file;
 	} In;
-	struct C {	/* -C */
+	struct GRDREDPOL_C {	/* -C */
+		bool active;
 		bool use_igrf;
 		bool const_f;
 		double	dec;
 		double	dip;
 	} C;
-	struct E {	/* -E */
+	struct GRDREDPOL_E {	/* -E */
 		bool active;
 		bool dip_grd_only;
 		bool dip_dec_grd;
 		char *decfile;
 		char *dipfile;
 	} E;
-	struct F {	/* -F */
+	struct GRDREDPOL_F {	/* -F */
 		bool active;
 		unsigned int	ncoef_row;
 		unsigned int	ncoef_col;
 		unsigned int	compute_n;	/* Compute ncoef_col */
 		double	width;
 	} F;
-	struct G {	/* -G<file> */
+	struct GRDREDPOL_G {	/* -G<file> */
 		bool active;
 		char	*file;
 	} G;
-	struct M {	/* -M */
+	struct GRDREDPOL_M {	/* -M */
+		bool active;
 		bool pad_zero;
 		bool mirror;
 	} M;
-	struct N {	/* -N */
+	struct GRDREDPOL_N {	/* -N */
 		bool active;
 	} N;
-	struct S {	/* -S, size of working grid */
+	struct GRDREDPOL_S {	/* -S, size of working grid */
+		bool active;
 		unsigned int	n_columns;
 		unsigned int	n_rows;
 	} S;
-	struct T {	/* -T */
+	struct GRDREDPOL_T {	/* -T */
+		bool active;
 		double	year;
 	} T;
-	struct W {	/* -W */
+	struct GRDREDPOL_W {	/* -W */
 		double	wid;
+		bool active;
 	} W;
-	struct Z {	/* -Z */
+	struct GRDREDPOL_Z {	/* -Z */
 		bool active;
 		char	*file;
 	} Z;
@@ -96,15 +101,14 @@ struct REDPOL_CTRL {
 #define ij0_data(Ctrl,i,j) ((Ctrl->S.n_columns+Ctrl->F.ncoef_col-1)*(i)+(j))
 #define ij_mn(Ctrl,i,j) (Ctrl->F.ncoef_row*(j)+(i))
 
-GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
-	struct REDPOL_CTRL *C;
-	
-	C = gmt_M_memory (GMT, NULL, 1, struct REDPOL_CTRL);
-	
+static void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
+	struct GRDREDPOL_CTRL *C;
+
+	C = gmt_M_memory (GMT, NULL, 1, struct GRDREDPOL_CTRL);
+
 	/* Initialize values whose defaults are not 0/false/NULL */
 	C->C.use_igrf = true;
 	C->M.pad_zero = true;
-	C->N.active = true;
 	C->F.ncoef_row = 25;
 	C->F.ncoef_col = 25;
 	C->T.year = 2000;
@@ -112,19 +116,19 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 	return (C);
 }
 
-GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct REDPOL_CTRL *C) {	/* Deallocate control structure */
+static void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDREDPOL_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
-	gmt_M_str_free (C->In.file);	
-	gmt_M_str_free (C->G.file);	
-	gmt_M_str_free (C->E.dipfile);	
-	gmt_M_str_free (C->E.decfile);	
-	gmt_M_str_free (C->Z.file);	
-	gmt_M_free (GMT, C);	
+	gmt_M_str_free (C->In.file);
+	gmt_M_str_free (C->G.file);
+	gmt_M_str_free (C->E.dipfile);
+	gmt_M_str_free (C->E.decfile);
+	gmt_M_str_free (C->Z.file);
+	gmt_M_free (GMT, C);
 }
 
-GMT_LOCAL void rtp_filt_colinear (int i, int j, int n21, double *gxr,double *gxi, double *gxar, 
-		double *gxai, double *gxbr, double *gxbi, double *gxgr, double *gxgi, double u, 
-		double v, double alfa, double beta, double gama, struct REDPOL_CTRL *Ctrl) {
+GMT_LOCAL void grdgravmag3d_rtp_filt_colinear (int i, int j, int n21, double *gxr,double *gxi, double *gxar,
+		double *gxai, double *gxbr, double *gxbi, double *gxgr, double *gxgi, double u,
+		double v, double alfa, double beta, double gama, struct GRDREDPOL_CTRL *Ctrl) {
 
 	uint64_t ij = ij_mn(Ctrl,i,j-n21+1);
 	double ro, ro2, ro3, ro4, ro5, alfa_u, beta_v, gama_ro, gama_ro_2;
@@ -139,14 +143,14 @@ GMT_LOCAL void rtp_filt_colinear (int i, int j, int n21, double *gxr,double *gxi
 
 	t2 = 1 / ((alfa_u_beta_v_2 + gama_ro_2) * (alfa_u_beta_v_2 + gama_ro_2));
 	t3 = t2 / (alfa_u_beta_v_2 + gama_ro_2);
-          					 
+
 	rnr = (gama_ro_2 - alfa_u_beta_v_2) * ro2;
 	rni = 2 * gama_ro * alfa_u_beta_v * ro2;
 
 	gxr[ij] = rnr * t2;
 	gxi[ij] = rni * t2;
 
-	if (Ctrl->N.active) {		/* Use the Taylor expansion */
+	if (!Ctrl->N.active) {		/* Use the Taylor expansion */
 		gxar[ij] = -2*alfa_u_beta_v*u*ro2 * t2 - 4*(gama_ro_2-alfa_u_beta_v_2)*ro2*alfa_u_beta_v*u * t3;
 		gxai[ij] = 2*gama*ro3*u * t2 - 8*gama*ro3*alfa_u_beta_v_2*u * t3;
 		gxbr[ij] = -2*alfa_u_beta_v*v*ro2 * t2 - 4*(gama_ro_2-alfa_u_beta_v_2)*ro2*alfa_u_beta_v*v * t3;
@@ -157,10 +161,10 @@ GMT_LOCAL void rtp_filt_colinear (int i, int j, int n21, double *gxr,double *gxi
 }
 
 
-GMT_LOCAL void rtp_filt_NOTcolinear (int i, int j, int n21, double *gxr, double *gxi, double *gxar, 
-		double *gxai, double *gxbr, double *gxbi, double *gxgr, double *gxgi, double *gxtr, 
-		double *gxti, double *gxmr, double *gxmi, double *gxnr, double *gxni, double u, double v, double alfa, 
-		double beta, double gama, double tau, double mu, double nu, struct REDPOL_CTRL *Ctrl) {
+GMT_LOCAL void grdgravmag3d_rtp_filt_not_colinear (int i, int j, int n21, double *gxr, double *gxi, double *gxar,
+		double *gxai, double *gxbr, double *gxbi, double *gxgr, double *gxgi, double *gxtr,
+		double *gxti, double *gxmr, double *gxmi, double *gxnr, double *gxni, double u, double v, double alfa,
+		double beta, double gama, double tau, double mu, double nu, struct GRDREDPOL_CTRL *Ctrl) {
 
 	uint64_t ij = ij_mn(Ctrl,i,j-n21+1);
 	double ro, ro2, ro3, ro4, ro5, alfa_u, beta_v, gama_ro, gama_ro_2;
@@ -195,7 +199,7 @@ GMT_LOCAL void rtp_filt_NOTcolinear (int i, int j, int n21, double *gxr, double
 	gxr[ij] = rnr / den_r;
 	gxi[ij] = rni / den_r;
 
-	if (Ctrl->N.active) {		/* Use the Taylor expansion */
+	if (!Ctrl->N.active) {		/* Use the Taylor expansion */
 		gxar[ij] = -u*tau_u_mu_v*ro2/den_r -2*(-alfa_u_beta_v_tau_u_mu_v+gama*ro2*nu)*ro2*alfa_u_beta_v*u/den_i1;
 		gxai[ij] = u*nu*ro3/den_r -2*(alfa_u_beta_v_nu+tau_u_mu_v_gama)*ro3*alfa_u_beta_v*u/den_i1;
 		gxbr[ij] = -v*tau_u_mu_v*ro2/den_r -2*(-alfa_u_beta_v_tau_u_mu_v+gama*ro2*nu)*ro2*alfa_u_beta_v*v/den_i1;
@@ -211,7 +215,7 @@ GMT_LOCAL void rtp_filt_NOTcolinear (int i, int j, int n21, double *gxr, double
 	}
 }
 
-GMT_LOCAL void mirror_edges (float *grid, int nc, int i_data_start, int j_data_start, struct REDPOL_CTRL *Ctrl) {
+GMT_LOCAL void grdgravmag3d_mirror_edges (gmt_grdfloat *grid, int nc, int i_data_start, int j_data_start, struct GRDREDPOL_CTRL *Ctrl) {
 	/* This routine mirrors or replicates the West and East borders j_data_start times
 	   and the South and North borders by i_data_start times.
 	   nc	is the total number of columns by which the grid is extended
@@ -222,11 +226,11 @@ GMT_LOCAL void mirror_edges (float *grid, int nc, int i_data_start, int j_data_s
 
 	upper_ny = Ctrl->S.n_rows+i_data_start;
 	for (j = 1; j <= j_data_start; j++) {	/* COLUMNS */
-		jww = j_data_start-j;		/* Minimum Outside xmin and aproaching West border  */
-		jee = Ctrl->S.n_columns + j_data_start + j-1;	/* Minimum Outside xmax and aproching East border  */
+		jww = j_data_start-j;		/* Minimum Outside xmin and approaching West border  */
+		jee = Ctrl->S.n_columns + j_data_start + j-1;	/* Minimum Outside xmax and approaching East border  */
 		if (Ctrl->M.mirror) {
-			jwe = j_data_start+j;			/* Minimum Inside xmin and aproaching center  */
-			jew = Ctrl->S.n_columns + j_data_start - j-1;	/* Minimum Inside xmax and aproching center  */
+			jwe = j_data_start+j;			/* Minimum Inside xmin and approaching center  */
+			jew = Ctrl->S.n_columns + j_data_start - j-1;	/* Minimum Inside xmax and approaching center  */
 		}
 		else {
 			jwe = j_data_start;			/* West border */
@@ -242,10 +246,10 @@ GMT_LOCAL void mirror_edges (float *grid, int nc, int i_data_start, int j_data_s
 
 	upper_nx = Ctrl->S.n_columns + nc;
 	for (i = 0; i < i_data_start; i++) {	/* ROWS */
-		iss = Ctrl->S.n_rows+i_data_start+i;	/* Minimum Outside ymin and aproaching South border  */
+		iss = Ctrl->S.n_rows+i_data_start+i;	/* Minimum Outside ymin and approaching South border  */
 		if (Ctrl->M.mirror) {
-			ins = 2*i_data_start - i;		/* Maximum Inside ymax and aproaching North border */
-			isn = Ctrl->S.n_rows+i_data_start-2-i;	/* Minimum Inside ymin and aproaching center  */
+			ins = 2*i_data_start - i;		/* Maximum Inside ymax and approaching North border */
+			isn = Ctrl->S.n_rows+i_data_start-2-i;	/* Minimum Inside ymin and approaching center  */
 		}
 		else {
 			ins = i_data_start;			/* North border */
@@ -258,7 +262,7 @@ GMT_LOCAL void mirror_edges (float *grid, int nc, int i_data_start, int j_data_s
 	}
 }
 
-GMT_LOCAL void tfpoeq(double *w, int m, int n, double *greel, double *gim, 
+GMT_LOCAL void grdgravmag3d_tfpoeq(double *w, int m, int n, double *greel, double *gim,
 	    double *cosphi, double *sinphi, double *cospsi, double *sinpsi) {
     /* Initialized data */
 
@@ -279,7 +283,7 @@ GMT_LOCAL void tfpoeq(double *w, int m, int n, double *greel, double *gim,
 /*          - M/N => NUMBER OF LINES/COLUMN OF THE FILTER */
 
 /*     OUTPUT ARGUMENTS DESCRIPTION: */
-/*          - W => AN ARRAY WHICH CONTAINS THE FILTER COEFICIENTS STORED */
+/*          - W => AN ARRAY WHICH CONTAINS THE FILTER COEFFICIENTS STORED */
 /*               COLUMNWISE */
 
 /*     REMARKS: */
@@ -368,11 +372,11 @@ L4:
 }
 
 
-GMT_LOCAL int igrf10syn (struct GMT_CTRL *C, int isv, double date, int itype, double alt, double elong, double lat, double *out) {
+GMT_LOCAL int grdgravmag3d_igrf10syn (struct GMT_CTRL *C, int isv, double date, int itype, double alt, double elong, double lat, double *out) {
  /*     This is a synthesis routine for the 10th generation IGRF as agreed
   *     in December 2004 by IAGA Working Group V-MOD. It is valid 1900.0 to
   *     2010.0 inclusive. Values for dates from 1945.0 to 2000.0 inclusive are
-  *     definitve, otherwise they are non-definitive.
+  *     definitive, otherwise they are non-definitive.
   *   INPUT
   *     isv   = 0 if main-field values are required
   *     isv   = 1 if secular variation values are required
@@ -407,19 +411,19 @@ GMT_LOCAL int igrf10syn (struct GMT_CTRL *C, int isv, double date, int itype, do
   *
   *	Joaquim Luis 1-MARS-2005
   *	Converted to C (with help of f2c, which explains the ugliness)
-  *     1995.0 coefficients as published in igrf9coeffs.xls and igrf10coeffs.xls 
-  *     used - (Kimmo Korhonen spotted 1 nT difference in 11 coefficients) 
-  *     Susan Macmillan July 2005 (PW update Oct 2006) 
+  *     1995.0 coefficients as published in igrf9coeffs.xls and igrf10coeffs.xls
+  *     used - (Kimmo Korhonen spotted 1 nT difference in 11 coefficients)
+  *     Susan Macmillan July 2005 (PW update Oct 2006)
   *
   *	Joaquim Luis 21-JAN-2010
   *	Updated for IGRF 11th generation
   */
 
-	struct IGRF {
+	struct GRDREDPOL_IGRF {
 		double e_1[3450];
 	};
      /* Initialized data */
-     static struct IGRF equiv_22 = {
+     static struct GRDREDPOL_IGRF equiv_22 = {
        {-31543.,-2298., 5922., -677., 2905.,-1061.,  924., 1121., /* g0 (1900) */
          1022.,-1469., -330., 1256.,    3.,  572.,  523.,  876.,
           628.,  195.,  660.,  -69., -361., -210.,  134.,  -75.,
@@ -883,12 +887,12 @@ GMT_LOCAL int igrf10syn (struct GMT_CTRL *C, int isv, double date, int itype, do
 	double rr, st, one, gmm, rho, two, three, ratio;
 	double p[105], q[105], r, t, a2, b2;
 	double H, F, X = 0, Y = 0, Z = 0, dec, dip;
-	
+
 	if (date < 1900.0 || date > 2020.0) {
-		GMT_Report (C->parent, GMT_MSG_NORMAL, "Your date (%g) is outside valid extrapolated range for IGRF (1900-2020)\n", date);
+		GMT_Report (C->parent, GMT_MSG_ERROR, "Your date (%g) is outside valid extrapolated range for IGRF (1900-2020)\n", date);
 		return (true);
 	}
-	
+
 	if (date < 2015.) {
 		t = 0.2 * (date - 1900.);
 		ll = (int) t;
@@ -1019,38 +1023,51 @@ GMT_LOCAL int igrf10syn (struct GMT_CTRL *C, int isv, double date, int itype, do
 	out[2] = X;		out[3] = Y;
 	out[4] = Z;
 	out[5] = dec;		out[6] = dip;
-	
+
 	return (GMT_NOERROR);
 }
 
-GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+static int usage (struct GMTAPI_CTRL *API, int level) {
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdredpol <anomgrid> -G<rtp_grdfile> [-C<dec>/<dip>] [-Ei<dip_grd>] [-Ee<dec_grd>] [-F<m>/<n>]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-M<m|r>] [-N] [-W<win_width>] [%s] [-T<year>] [-Z<filterfile>]\n\t[%s]\n\n",
-				GMT_Rgeo_OPT, GMT_V_OPT, GMT_n_OPT);
+	GMT_Usage (API, 0, "usage: %s %s -G<rtp_grdfile> [-C<dec>/<dip>] [-Ei|d<grid>] [-F<m>/<n>] "
+		"[-Mm|r] [-N] [-T<year>] [%s] [%s] [-W<win_width>] [-Z<filterfile>] [%s] [%s]\n",
+				name, GMT_INGRID, GMT_Rgeo_OPT, GMT_V_OPT, GMT_n_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
-                
-	GMT_Message (API, GMT_TIME_NONE, "\t<anomgrid> is the input grdfile with the magnetic anomaly.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-G Sets filename for output grid with the RTP solution.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-C Sets<dec>/<dip> and uses this constant values in the RTP procedure.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-Ei grid with the magnetization inclination [default: use IGRF].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-Ed grid with the magnetization declination [default: use IGRF].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-F Sets <m>/<n> filter widths [25x25].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-M Sets boundary conditions. m|r stands for mirror or replicate edges (Default is zero padding).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-N Do NOT use Taylor expansion.\n");
-	GMT_Option  (API, "R");
-	GMT_Message (API, GMT_TIME_NONE, "\t-T Sets year used by the IGRF routine to compute the various DECs & DIPs [default: 2000].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-W Sets window width in degrees [5].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-Z Write filter file <filterfile> to disk.\n");
-	GMT_Option  (API, "V,n,.");
-	
+
+	GMT_Message (API, GMT_TIME_NONE, "  REQUIRED ARGUMENTS:\n");
+	gmt_ingrid_syntax (API, 0, "Name of grid with the magnetic anomaly");
+	GMT_Usage (API, 1, "\n-G<rtp_grdfile>");
+	GMT_Usage (API, -2, "Set filename for output grid with the RTP solution.");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
+	GMT_Usage (API, 1, "\n-C<dec>/<dip>");
+	GMT_Usage (API, -2, "Set <dec>/<dip> and uses this constant values in the RTP procedure.");
+	GMT_Usage (API, 1, "\n-Ei|d<grid>");
+	GMT_Usage (API, -2, "Specify input grids for inclination and/or declination:");
+	GMT_Usage (API, 3, "i: Append file name for magnetization inclination [default uses IGRF].");
+	GMT_Usage (API, 3, "d: Append file name for the magnetization declination [default uses IGRF].");
+	GMT_Usage (API, 1, "\n-F<m>/<n>");
+	GMT_Usage (API, -2, "Set <m>/<n> filter widths [25/25].");
+	GMT_Usage (API, 1, "\n-Mm|r");
+	GMT_Usage (API, -2, "Specify optional boundary conditions [Default is zero padding]:");
+	GMT_Usage (API, 3, "m: mirror boundary condition.");
+	GMT_Usage (API, 3, "r: Replicate edges.");
+	GMT_Usage (API, 1, "\n-N Do NOT use Taylor expansion.");
+	GMT_Option (API, "R");
+	GMT_Usage (API, 1, "\n-T<year>");
+	GMT_Usage (API, -2, "Set year used by the IGRF routine to compute the various DECs & DIPs [default is 2000].");
+	GMT_Option  (API, "V");
+	GMT_Usage (API, 1, "\n-W<win_width>");
+	GMT_Usage (API, -2, "Set window width in degrees [5].");
+	GMT_Usage (API, 1, "\n-Z<filterfile>");
+	GMT_Usage (API, -2, "Write filter file <filterfile> to disk.");
+	GMT_Option  (API, "n,.");
+
 	return (GMT_MODULE_USAGE);
 }
 
-GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct REDPOL_CTRL *Ctrl, struct GMT_OPTION *options) {
+static int parse (struct GMT_CTRL *GMT, struct GRDREDPOL_CTRL *Ctrl, struct GMT_OPTION *options) {
 	/* This parses the options provided to grdredpol and sets parameters in Ctrl.
 	 * Note Ctrl has already been initialized and non-zero default values set.
 	 * Any GMT common options will override values set previously by other commands.
@@ -1058,7 +1075,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct REDPOL_CTRL *Ctrl, struct GMT_
 	 * returned when registering these sources/destinations with the API.
 	 */
 
-	unsigned int n_errors = 0, n_files = 0, pos = 0;
+	unsigned int n_errors = 0, pos = 0;
 	int    j;
 	char   p[GMT_LEN256] = {""};
 	struct GMT_OPTION *opt = NULL;
@@ -1068,13 +1085,14 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct REDPOL_CTRL *Ctrl, struct GMT_
 
 		switch (opt->option) {
 			case '<':	/* Input file (only one is accepted) */
-				Ctrl->In.active = true;
-				if (n_files++ == 0) Ctrl->In.file = strdup (opt->arg);
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->In.active);
+				n_errors += gmt_get_required_file (GMT, opt->arg, opt->option, 0, GMT_IS_GRID, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->In.file));
 				break;
 
 			/* Processes program-specific parameters */
 
 			case 'C':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->C.active);
 				sscanf (opt->arg, "%lf/%lf", &Ctrl->C.dec, &Ctrl->C.dip);
 				Ctrl->C.dec *= D2R;
 				Ctrl->C.dip *= D2R;
@@ -1082,7 +1100,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct REDPOL_CTRL *Ctrl, struct GMT_
 				Ctrl->C.use_igrf = false;
 				break;
 			case 'E':		/* -Ei<dip_grid> -Ee<dec_grid> */
-				Ctrl->E.active = true;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->E.active);
 				Ctrl->C.use_igrf = false;
 				Ctrl->E.dip_grd_only = true;
 
@@ -1106,7 +1124,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct REDPOL_CTRL *Ctrl, struct GMT_
 								Ctrl->E.dip_dec_grd = true;
 								break;
 							default:
-								GMT_Report (API, GMT_MSG_NORMAL, "Syntax error using option -E\n");
+								GMT_Report (API, GMT_MSG_ERROR, "Syntax error using option -E\n");
 								n_errors++;
 								break;
 						}
@@ -1115,24 +1133,24 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct REDPOL_CTRL *Ctrl, struct GMT_
 				}
 				break;
 			case 'F':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->F.active);
 				j = sscanf (opt->arg, "%d/%d", &Ctrl->F.ncoef_row, &Ctrl->F.ncoef_col);
 				if (j == 1) Ctrl->F.compute_n = true;	/* Case of only one filter dimension was given */
 				if (Ctrl->F.ncoef_row %2 != 1 || Ctrl->F.ncoef_col %2 != 1) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Error: number of filter coefficients must be odd\n");
+					GMT_Report (API, GMT_MSG_ERROR, "Number of filter coefficients must be odd\n");
 					n_errors++;
 				}
 				if (Ctrl->F.ncoef_row < 5 || Ctrl->F.ncoef_col < 5) {
-					GMT_Report (API, GMT_MSG_NORMAL, "That was a ridiculous number of filter coefficients\n");
+					GMT_Report (API, GMT_MSG_ERROR, "That was a ridiculous number of filter coefficients\n");
 					n_errors++;
 				}
 				break;
 			case 'G':
-				if ((Ctrl->G.active = gmt_check_filearg (GMT, 'G', opt->arg, GMT_OUT, GMT_IS_GRID)) != 0)
-					Ctrl->G.file = strdup (opt->arg);
-				else
-					n_errors++;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->G.active);
+				n_errors += gmt_get_required_file (GMT, opt->arg, opt->option, 0, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->G.file));
 				break;
 			case 'M':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->M.active);
 				Ctrl->M.pad_zero = false;
 				for (j = 0; opt->arg[j]; j++) {
 					if (opt->arg[j] == 'm')
@@ -1140,35 +1158,38 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct REDPOL_CTRL *Ctrl, struct GMT_
 					else if (opt->arg[j] == 'r')
 						Ctrl->M.mirror = false;
 					else {
-						GMT_Report (API, GMT_MSG_NORMAL, "Warning: Error using option -M (option ignored)\n");
+						GMT_Report (API, GMT_MSG_ERROR, "Failure while using option -M (option ignored)\n");
 						Ctrl->M.pad_zero = true;
 					}
 				}
 				break;
 			case 'N':
-				Ctrl->N.active = false;
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->N.active);
+				n_errors += gmt_get_no_argument (GMT, opt->arg, opt->option, 0);
 				break;
 			case 'T':
-				sscanf (opt->arg, "%lf", &Ctrl->T.year);
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->T.active);
+				n_errors += gmt_get_required_double (GMT, opt->arg, opt->option, 0, &Ctrl->T.year);
 				break;
 			case 'W':
-				sscanf (opt->arg, "%lf", &Ctrl->W.wid);
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->W.active);
+				n_errors += gmt_get_required_double (GMT, opt->arg, opt->option, 0, &Ctrl->W.wid);
 				break;
 			case 'Z':
-				Ctrl->Z.active = true;
-				Ctrl->Z.file = strdup (opt->arg);
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->Z.active);
+				n_errors += gmt_get_required_file (GMT, opt->arg, opt->option, 0, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->Z.file));
 				break;
 			default:	/* Report bad options */
-				n_errors += gmt_default_error (GMT, opt->option);
+				n_errors += gmt_default_option_error (GMT, opt);
 				break;
 		}
 	}
 
-	n_errors += gmt_M_check_condition (GMT, !Ctrl->In.file, "Syntax error: Must specify input file\n");
-	n_errors += gmt_M_check_condition (GMT, !Ctrl->G.file, "Syntax error -G option: Must specify output file\n");
+	n_errors += gmt_M_check_condition (GMT, !Ctrl->In.file, "Must specify input file\n");
+	n_errors += gmt_M_check_condition (GMT, !Ctrl->G.file, "Option -G: Must specify output file\n");
 
-	if (Ctrl->C.const_f && Ctrl->C.use_igrf) {	
-		GMT_Report (API, GMT_MSG_NORMAL, "Warning: -E option overrides -C\n");
+	if (Ctrl->C.const_f && Ctrl->C.use_igrf) {
+		GMT_Report (API, GMT_MSG_ERROR, "-E option overrides -C\n");
 		Ctrl->C.const_f = false;
 	}
 
@@ -1178,10 +1199,11 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct REDPOL_CTRL *Ctrl, struct GMT_
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
-int GMT_grdredpol (void *V_API, int mode, void *args) {
+EXTERN_MSC int GMT_grdredpol (void *V_API, int mode, void *args) {
 
 	bool wrote_one = false;
-	unsigned int i, j, row, col, nx_new, ny_new, one_or_zero, m21, n21, i2, j2;
+	openmp_int row, col;
+	unsigned int i, j, nx_new, ny_new, one_or_zero, m21, n21, i2, j2;
 	unsigned int k, l, i3, n_jlon, n_jlat, n_coef;
 	int error = 0;
 	uint64_t ij, jj;
@@ -1197,7 +1219,7 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 	double	dec_m, dip_m, tau1, mu1, nu1, dt = 0, dm = 0, dn = 0, tau = 0, mu = 0, nu = 0;
 	double	wesn_new[4], out_igrf[7];
 
-	struct	REDPOL_CTRL *Ctrl = NULL;
+	struct	GRDREDPOL_CTRL *Ctrl = NULL;
 	struct	GMT_GRID *Gin = NULL, *Gout = NULL, *Gdip = NULL, *Gdec = NULL, *Gfilt = NULL;
 	struct	GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct	GMT_OPTION *options = NULL;
@@ -1209,18 +1231,15 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) 
-		bailout (usage (API, GMT_USAGE));		/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) 
-		bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, NULL, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
-	
+
 	/*--------------------------- This is the grdredpol main code --------------------------*/
 
 	if ((Gin = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->In.file, NULL)) == NULL) /* Get header only */
@@ -1236,7 +1255,7 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 	GMT->current.io.pad[XLO] = GMT->current.io.pad[XHI] = n21-1;
 	GMT->current.io.pad[YLO] = GMT->current.io.pad[YHI] = m21-1;
 
-	if (!GMT->common.R.active[RSET]) 
+	if (!GMT->common.R.active[RSET])
 		gmt_M_memcpy (wesn_new, Gin->header->wesn, 4, double);
 	else
 		gmt_M_memcpy (wesn_new, GMT->common.R.wesn, 4, double);
@@ -1255,31 +1274,31 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 
 	if (GMT->common.R.active[RSET]) {
 		if (wesn_new[XLO] < Gin->header->wesn[XLO] || wesn_new[XHI] > Gin->header->wesn[XHI]) {
-			GMT_Report (API, GMT_MSG_NORMAL, " Selected region exceeds the X-boundaries of the grid file!\n");
+			GMT_Report (API, GMT_MSG_ERROR, " Selected region exceeds the X-boundaries of the grid file!\n");
 			return (GMT_RUNTIME_ERROR);
 		}
 		else if (wesn_new[YLO] < Gin->header->wesn[YLO] || wesn_new[YHI] > Gin->header->wesn[YHI]) {
-			GMT_Report (API, GMT_MSG_NORMAL, " Selected region exceeds the Y-boundaries of the grid file!\n");
+			GMT_Report (API, GMT_MSG_ERROR, " Selected region exceeds the Y-boundaries of the grid file!\n");
 			return (GMT_RUNTIME_ERROR);
 		}
 	}
 
 	if (!Ctrl->M.pad_zero)		/* That is, if we want edges reflected or replicated */
-		mirror_edges (Gin->data, Ctrl->F.ncoef_col-1, m21-1, n21-1, Ctrl);
+		grdgravmag3d_mirror_edges (Gin->data, Ctrl->F.ncoef_col-1, m21-1, n21-1, Ctrl);
 
 	/* Section to deal with possible external grids with dip and dec for interpolation */
 
 	if (Ctrl->E.dip_grd_only || Ctrl->E.dip_dec_grd) {
 		if ((Gdip = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->E.dipfile, NULL)) == NULL)	/* Get header only */
 			Return (API->error);
-	
+
 		if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, wesn_new, Ctrl->E.dipfile, Gdip) == NULL)
 			Return (API->error);
 	}
 	if (Ctrl->E.dip_dec_grd) {
 		if ((Gdec = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->E.decfile, NULL)) == NULL)
 			Return (API->error);
-	
+
 		if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, wesn_new, Ctrl->E.decfile, Gdec) == NULL)
 			Return (API->error);
 	}
@@ -1305,7 +1324,7 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 	ftlon  = gmt_M_memory (GMT, NULL, Gin->header->n_columns, double);
 	ftlat  = gmt_M_memory (GMT, NULL, Gin->header->n_rows, double);
 
-	if ((Ctrl->E.dip_grd_only || Ctrl->E.dip_dec_grd)) { 
+	if ((Ctrl->E.dip_grd_only || Ctrl->E.dip_dec_grd)) {
 		gxtr = gmt_M_memory (GMT, NULL, n_coef, double);
 		gxti = gmt_M_memory (GMT, NULL, n_coef, double);
 		gxmr = gmt_M_memory (GMT, NULL, n_coef, double);
@@ -1318,8 +1337,8 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 	}
 
 	/* Generate vectors of lon & lats */
-	for (col = 0; col < Gin->header->n_columns; col++) ftlon[col] = gmt_M_grd_col_to_x (GMT, col, Gin->header);
-	for (row = 0; row < Gin->header->n_rows; row++) ftlat[row] = gmt_M_grd_row_to_y (GMT, row, Gin->header);
+	for (col = 0; col < (openmp_int)Gin->header->n_columns; col++) ftlon[col] = gmt_M_grd_col_to_x (GMT, col, Gin->header);
+	for (row = 0; row < (openmp_int)Gin->header->n_rows; row++) ftlat[row] = gmt_M_grd_row_to_y (GMT, row, Gin->header);
 
 	n_jlon = urint ((Gin->header->wesn[XHI] - Gin->header->wesn[XLO]) / Ctrl->W.wid) + 1;
 	n_jlat = urint ((Gin->header->wesn[YHI] - Gin->header->wesn[YLO]) / Ctrl->W.wid) + 1;
@@ -1329,19 +1348,19 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 		beta =  cos(Ctrl->C.dip) * sin(Ctrl->C.dec);
 		gama = -sin(Ctrl->C.dip);
 	}
-	
+
 	fi  = TWO_PI / Ctrl->F.ncoef_row;
 	psi = TWO_PI / Ctrl->F.ncoef_col;
 
 	if ((Gout = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, wesn_new, Gin->header->inc, \
 	                             Gin->header->registration, GMT_NOTSET, NULL)) == NULL) Return (API->error);
-					
+
 	if (Ctrl->Z.active) {		/* Create one grid to hold the filter coefficients */
 		double wesn[4], inc[2];
 		wesn[XLO] = 1;	wesn[XHI] = (double)Ctrl->F.ncoef_col;
 		wesn[YLO] = 1;	wesn[YHI] = (double)Ctrl->F.ncoef_row;
 		inc[GMT_X] = inc[GMT_Y] = 1;
-		
+
 		if ((Gfilt = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, wesn, inc,
 		                              GMT_GRID_PIXEL_REG, 0, NULL)) == NULL) Return (API->error);
 		strcpy (Gfilt->header->title, "Reduction To the Pole filter");
@@ -1351,8 +1370,8 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 
 	for (l = 0; l < n_jlat; l++) {		/* Main loop over the moving windows */
 		for (k = 0; k < n_jlon; k++) {
-			sloni = Gin->header->wesn[XLO] + k * Ctrl->W.wid; 
-			slati = Gin->header->wesn[YHI] - (l+1) * Ctrl->W.wid; 
+			sloni = Gin->header->wesn[XLO] + k * Ctrl->W.wid;
+			slati = Gin->header->wesn[YHI] - (l+1) * Ctrl->W.wid;
 			slonf = sloni + Ctrl->W.wid;
 			slatf = slati + Ctrl->W.wid;
 			slonm = (sloni + slonf) / 2;
@@ -1365,7 +1384,7 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 				n21 = (Ctrl->F.ncoef_row+1) / 2;
 			}
 			/* Compute dec and dip at the central point of the moving window */
-			igrf10syn(GMT, 0, Ctrl->T.year, 1, 0, slonm, slatm, out_igrf); 
+			grdgravmag3d_igrf10syn(GMT, 0, Ctrl->T.year, 1, 0, slonm, slatm, out_igrf);
 			if (!Ctrl->C.const_f) {
 				Ctrl->C.dec = out_igrf[5] * D2R;
 				Ctrl->C.dip = out_igrf[6] * D2R;
@@ -1389,8 +1408,8 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 					nu  = -sin(dip_m);
 				}
 			}
-			if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE))
-				GMT_Report (API, GMT_MSG_VERBOSE, "Dec %5.1f  Dip %5.1f  Bin_lon %6.1f  Bin_lat %5.1f\r", 
+			if (gmt_M_is_verbose (GMT, GMT_MSG_INFORMATION))
+				GMT_Report (API, GMT_MSG_INFORMATION, "Dec %5.1f  Dip %5.1f  Bin_lon %6.1f  Bin_lat %5.1f\r",
 					    Ctrl->C.dec/D2R, Ctrl->C.dip/D2R, slonm, slatm);
 
 			/* Compute the filter coefficients in the frequency domain */
@@ -1401,33 +1420,33 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 					u = r * fi;		v = s * psi;
 
 					if (!(Ctrl->E.dip_grd_only || Ctrl->E.dip_dec_grd)) {
-						rtp_filt_colinear(i,j,n21,gxr,gxi,gxar,gxai,gxbr,gxbi,gxgr,gxgi,
+						grdgravmag3d_rtp_filt_colinear(i,j,n21,gxr,gxi,gxar,gxai,gxbr,gxbi,gxgr,gxgi,
 						                  u,v,alfa,beta,gama, Ctrl);
 					}
 					else {		/* Case more general. alfa, beta, gama, tau, mu e nu */
-						rtp_filt_NOTcolinear(i,j,n21,gxr,gxi,gxar,gxai,gxbr,gxbi,gxgr,gxgi,gxtr,gxti,
+						grdgravmag3d_rtp_filt_not_colinear(i,j,n21,gxr,gxi,gxar,gxai,gxbr,gxbi,gxgr,gxgi,gxtr,gxti,
 						                     gxmr,gxmi,gxnr,gxni,u,v,alfa,beta,gama,tau,mu,nu, Ctrl);
 					}
 				}
 			}
-			gxr[ij_mn(Ctrl,m21-1,0)]  = 1;	gxi[ij_mn(Ctrl,m21-1,0)]  = 0; 
-			gxar[ij_mn(Ctrl,m21-1,0)] = 0;	gxai[ij_mn(Ctrl,m21-1,0)] = 0; 
-			gxbr[ij_mn(Ctrl,m21-1,0)] = 0;	gxbi[ij_mn(Ctrl,m21-1,0)] = 0; 
-			gxgr[ij_mn(Ctrl,m21-1,0)] = 0;	gxgi[ij_mn(Ctrl,m21-1,0)] = 0; 
+			gxr[ij_mn(Ctrl,m21-1,0)]  = 1;	gxi[ij_mn(Ctrl,m21-1,0)]  = 0;
+			gxar[ij_mn(Ctrl,m21-1,0)] = 0;	gxai[ij_mn(Ctrl,m21-1,0)] = 0;
+			gxbr[ij_mn(Ctrl,m21-1,0)] = 0;	gxbi[ij_mn(Ctrl,m21-1,0)] = 0;
+			gxgr[ij_mn(Ctrl,m21-1,0)] = 0;	gxgi[ij_mn(Ctrl,m21-1,0)] = 0;
 
 			/* Compute iFT of the filter */
-			tfpoeq(fxr, Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxr,gxi,   cosphi,sinphi,cospsi,sinpsi); 
-			tfpoeq(fxar,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxar,gxai, cosphi,sinphi,cospsi,sinpsi); 
-			tfpoeq(fxbr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxbr,gxbi, cosphi,sinphi,cospsi,sinpsi); 
-			tfpoeq(fxgr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxgr,gxgi, cosphi,sinphi,cospsi,sinpsi); 
+			grdgravmag3d_tfpoeq(fxr, Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxr,gxi,   cosphi,sinphi,cospsi,sinpsi);
+			grdgravmag3d_tfpoeq(fxar,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxar,gxai, cosphi,sinphi,cospsi,sinpsi);
+			grdgravmag3d_tfpoeq(fxbr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxbr,gxbi, cosphi,sinphi,cospsi,sinpsi);
+			grdgravmag3d_tfpoeq(fxgr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxgr,gxgi, cosphi,sinphi,cospsi,sinpsi);
 
-			if ((Ctrl->E.dip_grd_only || Ctrl->E.dip_dec_grd)) { 
-				gxtr[ij_mn(Ctrl,m21-1,0)] = 0;	gxti[ij_mn(Ctrl,m21-1,0)] = 0; 
-				gxmr[ij_mn(Ctrl,m21-1,0)] = 0;	gxmi[ij_mn(Ctrl,m21-1,0)] = 0; 
+			if ((Ctrl->E.dip_grd_only || Ctrl->E.dip_dec_grd)) {
+				gxtr[ij_mn(Ctrl,m21-1,0)] = 0;	gxti[ij_mn(Ctrl,m21-1,0)] = 0;
+				gxmr[ij_mn(Ctrl,m21-1,0)] = 0;	gxmi[ij_mn(Ctrl,m21-1,0)] = 0;
 				gxnr[ij_mn(Ctrl,m21-1,0)] = 0;	gxni[ij_mn(Ctrl,m21-1,0)] = 0;
-				tfpoeq(fxtr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxtr,gxti, cosphi,sinphi,cospsi,sinpsi); 
-				tfpoeq(fxmr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxmr,gxmi, cosphi,sinphi,cospsi,sinpsi); 
-				tfpoeq(fxnr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxnr,gxni, cosphi,sinphi,cospsi,sinpsi);
+				grdgravmag3d_tfpoeq(fxtr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxtr,gxti, cosphi,sinphi,cospsi,sinpsi);
+				grdgravmag3d_tfpoeq(fxmr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxmr,gxmi, cosphi,sinphi,cospsi,sinpsi);
+				grdgravmag3d_tfpoeq(fxnr,Ctrl->F.ncoef_row, Ctrl->F.ncoef_col,gxnr,gxni, cosphi,sinphi,cospsi,sinpsi);
 			}
 
 			/* Convolve filter with input data that is inside current window (plus what filter width imposes) */
@@ -1435,9 +1454,9 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 				if (ftlat[row] < slati || ftlat[row] > slatf) continue;		/* Current point outside WOI */
 				gmt_M_col_loop (GMT, Gout,row,col,ij) {
 					if (ftlon[col] < sloni || ftlon[col] > slonf) continue;	/* Current point outside WOI */
-					/* Compute dec and dip at corrent point */
+					/* Compute dec and dip at current point */
 					if (!Ctrl->C.const_f) {		/* It means we need to get F (& M) vector parameters */
-						igrf10syn(GMT, 0, Ctrl->T.year, 1, 0, ftlon[col], ftlat[row], out_igrf); 
+						grdgravmag3d_igrf10syn(GMT, 0, Ctrl->T.year, 1, 0, ftlon[col], ftlat[row], out_igrf);
 						Ctrl->C.dec = out_igrf[5] * D2R;
 						Ctrl->C.dip = out_igrf[6] * D2R;
 						if (Ctrl->E.dip_grd_only) {
@@ -1461,12 +1480,12 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 						gama1 = -sin(Ctrl->C.dip);
 						da = alfa1 - alfa;	db = beta1 - beta;	dg = gama1 - gama;
 					}
-					if (!Ctrl->N.active)		/* Do not use the Taylor expansion (What's the interest?) */
+					if (Ctrl->N.active)		/* Do not use the Taylor expansion (What's the interest?) */
 						da = db = dg = dt = dm = dn = 0;
 
 					/* Rebuild the filter */
 					if (!Ctrl->C.const_f) {
-						if (Ctrl->C.use_igrf) { 
+						if (Ctrl->C.use_igrf) {
 							for (i2 = 0; i2 < n_coef; i2++)
 								fix[i2] = fxr[i2] + da * fxar[i2] + db * fxbr[i2] + dg * fxgr[i2];
 						}
@@ -1482,7 +1501,7 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 					if (Ctrl->Z.active && !wrote_one && l == 0 && k == 0) {
 						for (jj = i2 = 0; i2 < Ctrl->F.ncoef_row; i2++)		/* Remember, filter is columnwise */
 							for (j2 = 0; j2 < Ctrl->F.ncoef_col; j2++, jj++)
-								Gfilt->data[jj] = (float)fix[ij_mn(Ctrl,i2,j2)];
+								Gfilt->data[jj] = (gmt_grdfloat)fix[ij_mn(Ctrl,i2,j2)];
 
 						wrote_one = true;
 					}
@@ -1494,13 +1513,13 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 							tmp_d += fix[ij_mn(Ctrl,i2,j2)] * Gin->data[ij0_data(Ctrl,i3,(col + j2))];
 					}
 
-					Gout->data[ij] = (float)tmp_d;
+					Gout->data[ij] = (gmt_grdfloat)tmp_d;
 				}
 			}
 		}
 	}
 
-	if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) GMT_Report (API, GMT_MSG_VERBOSE, "\n"); 
+	if (gmt_M_is_verbose (GMT, GMT_MSG_INFORMATION)) GMT_Report (API, GMT_MSG_LONG_VERBOSE, "\n");
 
 	gmt_M_free (GMT, cosphi);      gmt_M_free (GMT, sinphi);
 	gmt_M_free (GMT, cospsi);      gmt_M_free (GMT, sinpsi);
@@ -1522,7 +1541,7 @@ int GMT_grdredpol (void *V_API, int mode, void *args) {
 		gmt_M_free (GMT, fxnr);
 	}
 
-	strcpy (Gout->header->title, "Anomaly reducted to the pole");
+	strcpy (Gout->header->title, "Anomaly reduced to the pole");
 	strcpy (Gout->header->z_units, "nT");
 
 	if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Gout)) Return (API->error);
